@@ -7,13 +7,18 @@ import string
 from reportlab.lib import colors
 from reportlab.lib.utils import simpleSplit, _simpleSplit, isBytes
 from reportlab.lib.validators import isNumber, isNumberOrNone, OneOf, isColorOrNone, isString, \
-        isTextAnchor, isBoxAnchor, isBoolean, NoneOr, isInstanceOf, isNoneOrString, isNoneOrCallable
+        isTextAnchor, isBoxAnchor, isBoolean, NoneOr, isInstanceOf, isNoneOrString, isNoneOrCallable, \
+        isSubclassOf
 from reportlab.lib.attrmap import *
 from reportlab.pdfbase.pdfmetrics import stringWidth, getAscentDescent, getFont, unicode2T1
 from reportlab.graphics.shapes import Drawing, Group, Circle, Rect, String, STATE_DEFAULTS
 from reportlab.graphics.shapes import _PATH_OP_ARG_COUNT, _PATH_OP_NAMES, definePath
 from reportlab.graphics.widgetbase import Widget, PropHolder
-from reportlab.graphics.shapes import _baseGFontName
+from reportlab.graphics.shapes import _baseGFontName, DirectDraw
+from reportlab.platypus import XPreformatted, Paragraph, Flowable
+from reportlab.lib.styles import ParagraphStyle, PropertySet
+from reportlab.lib.enums import TA_LEFT, TA_RIGHT, TA_CENTER
+_ta2al = dict(start=TA_LEFT,end=TA_RIGHT,middle=TA_CENTER)
 
 _gs = None
 _A2BA=  {
@@ -101,6 +106,11 @@ def _text2Path(text, x=0, y=0, fontName=_baseGFontName, fontSize=1000,
     return definePath(_text2PathDescription(text,x=x,y=y,fontName=fontName,
                     fontSize=fontSize,anchor=anchor,truncate=truncate,pathReverse=pathReverse),**kwds)
 
+try:
+    from rlextra.graphics.canvasadapter import DirectDrawFlowable
+except ImportError:
+    DirectDrawFlowable = None
+
 _BA2TA={'w':'start','nw':'start','sw':'start','e':'end', 'ne': 'end', 'se':'end', 'n':'middle','s':'middle','c':'middle'}
 class Label(Widget):
     """A text label to attach to something else, such as a chart axis.
@@ -141,6 +151,9 @@ class Label(Widget):
         bottomPadding = AttrMapValue(isNumber,desc='padding at bottom of box'),
         useAscentDescent = AttrMapValue(isBoolean,desc="If True then the font's Ascent & Descent will be used to compute default heights and baseline."),
         customDrawChanger = AttrMapValue(isNoneOrCallable,desc="An instance of CustomDrawChanger to modify the behavior at draw time", _advancedUsage=1),
+        ddf = AttrMapValue(NoneOr(isSubclassOf(DirectDraw),'NoneOrDirectDraw'),desc="A DirectDrawFlowable instance", _advancedUsage=1),
+        ddfKlass = AttrMapValue(NoneOr(isSubclassOf(Flowable),'NoneOrDirectDraw'),desc="A DirectDrawFlowable instance", _advancedUsage=1),
+        ddfStyle = AttrMapValue(NoneOr(isSubclassOf(PropertySet)),desc="A style for a ddfKlass or None", _advancedUsage=1),
         )
 
     def __init__(self,**kw):
@@ -173,6 +186,9 @@ class Label(Widget):
                 textAnchor = 'start',
                 visible = 1,
                 useAscentDescent = False,
+                ddf = DirectDrawFlowable,
+                ddfKlass = None,
+                ddfStyle = None,
                 )
 
     def setText(self, text):
@@ -252,23 +268,60 @@ class Label(Widget):
 
     def computeSize(self):
         # the thing will draw in its own coordinate system
-        self._lineWidths = []
-        self._lines = simpleSplit(self._text,self.fontName,self.fontSize,self.maxWidth)
-        if not self.width:
-            self._width = self.leftPadding+self.rightPadding
-            if self._lines:
-                self._lineWidths = [stringWidth(line,self.fontName,self.fontSize) for line in self._lines]
-                self._width += max(self._lineWidths)
+        ddfKlass = getattr(self,'ddfKlass',None)
+        if not ddfKlass:
+            self._lineWidths = []
+            self._lines = simpleSplit(self._text,self.fontName,self.fontSize,self.maxWidth)
+            if not self.width:
+                self._width = self.leftPadding+self.rightPadding
+                if self._lines:
+                    self._lineWidths = [stringWidth(line,self.fontName,self.fontSize) for line in self._lines]
+                    self._width += max(self._lineWidths)
+            else:
+                self._width = self.width
+            self._getBaseLineRatio()
+            if self.leading:
+                self._leading = self.leading
+            elif self.useAscentDescent:
+                self._leading = self._ascent - self._descent
+            else:
+                self._leading = self.fontSize*1.2
+            objH = self._leading*len(self._lines)
         else:
-            self._width = self.width
-        self._getBaseLineRatio()
-        if self.leading:
-            self._leading = self.leading
-        elif self.useAscentDescent:
-            self._leading = self._ascent - self._descent
-        else:
-            self._leading = self.fontSize*1.2
-        self._computeSizeEnd(self._leading*len(self._lines))
+            if self.ddf is None:
+                raise RuntimeError('DirectDrawFlowable class is not available you need the rlextra package as well as reportlab')
+            sty = dict(
+                    name='xlabel-generated',
+                    fontName=self.fontName,
+                    fontSize=self.fontSize,
+                    fillColor=self.fillColor,
+                    strokeColor=self.strokeColor,
+                    )
+            sty = self._style =  (ddfStyle.clone if self.ddfStyle else ParagraphStyle)(**sty)
+            self._getBaseLineRatio()
+            if self.useAscentDescent:
+                sty.autoLeading = True
+                sty.leading = self._ascent - self._descent
+            else:
+                sty.leading = self.leading if self.leading else self.fontSize*1.2
+            self._leading = sty.leading
+            ta = self._getTextAnchor()
+            aW = self.maxWidth or 0x7fffffff
+            if ta!='start':
+                sty.alignment = TA_LEFT
+                obj = ddfKlass(self._text,style=sty)
+                _, objH = obj.wrap(aW,0x7fffffff)
+                aW = self.maxWidth or obj._width_max
+            sty.alignment = _ta2al[ta]
+            self._ddfObj = obj = ddfKlass(self._text,style=sty)
+            _, objH = obj.wrap(aW,0x7fffffff)
+
+            if not self.width:
+                self._width = self.leftPadding+self.rightPadding
+                self._width += obj._width_max
+            else:
+                self._width = self.width
+        self._computeSizeEnd(objH)
 
     def _getTextAnchor(self):
         '''This can be overridden to allow special effects'''
@@ -285,14 +338,18 @@ class Label(Widget):
         g.translate(self.x + self.dx, self.y + self.dy)
         g.rotate(self.angle)
 
-        y = self._top - self._leading*self._baselineRatio
-        textAnchor = self._getTextAnchor()
-        if textAnchor == 'start':
+        ddfKlass = getattr(self,'ddfKlass',None)
+        if ddfKlass:
             x = self._left
-        elif textAnchor == 'middle':
-            x = self._left + self._ewidth*0.5
         else:
-            x = self._right
+            y = self._top - self._leading*self._baselineRatio
+            textAnchor = self._getTextAnchor()
+            if textAnchor == 'start':
+                x = self._left
+            elif textAnchor == 'middle':
+                x = self._left + self._ewidth*0.5
+            else:
+                x = self._right
 
         # paint box behind text just in case they
         # fill it
@@ -306,26 +363,32 @@ class Label(Widget):
                         fillColor=self.boxFillColor)
                         )
 
-        fillColor, fontName, fontSize = self.fillColor, self.fontName, self.fontSize
-        strokeColor, strokeWidth, leading = self.strokeColor, self.strokeWidth, self._leading
-        svgAttrs=getattr(self,'_svgAttrs',{})
-        if strokeColor:
-            for line in self._lines:
-                s = _text2Path(line, x, y, fontName, fontSize, textAnchor)
-                s.fillColor = fillColor
-                s.strokeColor = strokeColor
-                s.strokeWidth = strokeWidth
-                g.add(s)
-                y -= leading
+        if ddfKlass:
+            g1 = Group()
+            g1.translate(x,self._top-self._eheight)
+            g1.add(self.ddf(self._ddfObj))
+            g.add(g1)
         else:
-            for line in self._lines:
-                s = String(x, y, line, _svgAttrs=svgAttrs)
-                s.textAnchor = textAnchor
-                s.fontName = fontName
-                s.fontSize = fontSize
-                s.fillColor = fillColor
-                g.add(s)
-                y -= leading
+            fillColor, fontName, fontSize = self.fillColor, self.fontName, self.fontSize
+            strokeColor, strokeWidth, leading = self.strokeColor, self.strokeWidth, self._leading
+            svgAttrs=getattr(self,'_svgAttrs',{})
+            if strokeColor:
+                for line in self._lines:
+                    s = _text2Path(line, x, y, fontName, fontSize, textAnchor)
+                    s.fillColor = fillColor
+                    s.strokeColor = strokeColor
+                    s.strokeWidth = strokeWidth
+                    g.add(s)
+                    y -= leading
+            else:
+                for line in self._lines:
+                    s = String(x, y, line, _svgAttrs=svgAttrs)
+                    s.textAnchor = textAnchor
+                    s.fontName = fontName
+                    s.fontSize = fontSize
+                    s.fillColor = fillColor
+                    g.add(s)
+                    y -= leading
 
         return g
 
@@ -429,8 +492,8 @@ class PMVLabel(Label):
         BASE=Label,
         )
 
-    def __init__(self):
-        Label.__init__(self)
+    def __init__(self, **kwds):
+        Label.__init__(self, **kwds)
         self._pmv = 0
 
     def _getBoxAnchor(self):
@@ -457,8 +520,8 @@ class BarChartLabel(PMVLabel):
         boxTarget = AttrMapValue(OneOf('normal','anti','lo','hi','mid'),desc="one of ('normal','anti','lo','hi','mid')"),
         )
 
-    def __init__(self):
-        PMVLabel.__init__(self)
+    def __init__(self, **kwds):
+        PMVLabel.__init__(self, **kwds)
         self.lineStrokeWidth = 0
         self.lineStrokeColor = None
         self.fixedStart = self.fixedEnd = None
@@ -489,85 +552,81 @@ class RedNegativeChanger(CustomDrawChanger):
             obj.fillColor = self.fillColor
         return R
 
-from reportlab.platypus import XPreformatted, Paragraph
-from reportlab.lib.styles import ParagraphStyle
-from reportlab.lib.enums import TA_LEFT, TA_RIGHT, TA_CENTER
-_ta2al = dict(start=TA_LEFT,end=TA_RIGHT,middle=TA_RIGHT)
-try:
-    from rlextra.graphics.canvasadapter import DirectDrawFlowable
-except ImportError:
-    DirectDrawFlowable = None
-
 class XLabel(Label):
     '''like label but uses XPreFormatted/Paragraph to draw the _text'''
     _attrMap = AttrMap(BASE=Label,
             )
-
     def __init__(self,*args,**kwds):
-        self._flowableClass = kwds.pop('flowableClass',XPreformatted)
-        ddf = kwds.pop('directDrawClass',DirectDrawFlowable)
-        if ddf is None:
-            raise RuntimeError('DirectDrawFlowable class is not available you need the rlextra package as well as reportlab')
-        self._ddf = ddf
         Label.__init__(self,*args,**kwds)
+        self.ddfKlass = kwds.pop('flowableClass',XPreformatted)
+        self.ddf = kwds.pop('directDrawClass',self.ddf)
 
-    def computeSize(self):
-        # the thing will draw in its own coordinate system
-        self._lineWidths = []
-        sty = self._style = ParagraphStyle('xlabel-generated',
-                fontName=self.fontName,
-                fontSize=self.fontSize,
-                fillColor=self.fillColor,
-                strokeColor=self.strokeColor,
-                )
-        self._getBaseLineRatio()
-        if self.useAscentDescent:
-            sty.autoLeading = True
-            sty.leading = self._ascent - self._descent
-        else:
-            sty.leading = self.leading if self.leading else self.fontSize*1.2
-        self._leading = sty.leading
-        sty.alignment = _ta2al[self._getTextAnchor()]
-        self._obj = obj = self._flowableClass(self._text,style=sty)
-        _, objH = obj.wrap(self.maxWidth or 0x7fffffff,0x7fffffff)
+    if False:
+        def __init__(self,*args,**kwds):
+            self._flowableClass = kwds.pop('flowableClass',XPreformatted)
+            ddf = kwds.pop('directDrawClass',DirectDrawFlowable)
+            if ddf is None:
+                raise RuntimeError('DirectDrawFlowable class is not available you need the rlextra package as well as reportlab')
+            self._ddf = ddf
+            Label.__init__(self,*args,**kwds)
+        def computeSize(self):
+            # the thing will draw in its own coordinate system
+            self._lineWidths = []
+            sty = self._style = ParagraphStyle('xlabel-generated',
+                    fontName=self.fontName,
+                    fontSize=self.fontSize,
+                    fillColor=self.fillColor,
+                    strokeColor=self.strokeColor,
+                    )
+            self._getBaseLineRatio()
+            if self.useAscentDescent:
+                sty.autoLeading = True
+                sty.leading = self._ascent - self._descent
+            else:
+                sty.leading = self.leading if self.leading else self.fontSize*1.2
+            self._leading = sty.leading
+            ta = self._getTextAnchor()
+            aW = self.maxWidth or 0x7fffffff
+            if ta!='start':
+                sty.alignment = TA_LEFT
+                obj = self._flowableClass(self._text,style=sty)
+                _, objH = obj.wrap(aW,0x7fffffff)
+                aW = self.maxWidth or obj._width_max
+            sty.alignment = _ta2al[ta]
+            self._obj = obj = self._flowableClass(self._text,style=sty)
+            _, objH = obj.wrap(aW,0x7fffffff)
 
-        if not self.width:
-            self._width = self.leftPadding+self.rightPadding
-            self._width += self._obj._width_max
-        else:
-            self._width = self.width
-        self._computeSizeEnd(objH)
+            if not self.width:
+                self._width = self.leftPadding+self.rightPadding
+                self._width += self._obj._width_max
+            else:
+                self._width = self.width
+            self._computeSizeEnd(objH)
 
-    def _rawDraw(self):
-        _text = self._text
-        self._text = _text or ''
-        self.computeSize()
-        self._text = _text
-        g = Group()
-        g.translate(self.x + self.dx, self.y + self.dy)
-        g.rotate(self.angle)
+        def _rawDraw(self):
+            _text = self._text
+            self._text = _text or ''
+            self.computeSize()
+            self._text = _text
+            g = Group()
+            g.translate(self.x + self.dx, self.y + self.dy)
+            g.rotate(self.angle)
 
-        textAnchor = self._getTextAnchor()
-        if textAnchor == 'start':
             x = self._left
-        elif textAnchor == 'middle':
-            x = self._left + self._ewidth*0.5
-        else:
-            x = self._right
 
-        # paint box behind text just in case they
-        # fill it
-        if self.boxFillColor or (self.boxStrokeColor and self.boxStrokeWidth):
-            g.add(Rect( self._left-self.leftPadding,
-                        self._bottom-self.bottomPadding,
-                        self._width,
-                        self._height,
-                        strokeColor=self.boxStrokeColor,
-                        strokeWidth=self.boxStrokeWidth,
-                        fillColor=self.boxFillColor)
-                        )
-        g1 = Group()
-        g1.translate(x,self._top-self._eheight)
-        g1.add(self._ddf(self._obj))
-        g.add(g1)
-        return g
+            # paint box behind text just in case they
+            # fill it
+            if self.boxFillColor or (self.boxStrokeColor and self.boxStrokeWidth):
+                g.add(Rect( self._left-self.leftPadding,
+                            self._bottom-self.bottomPadding,
+                            self._width,
+                            self._height,
+                            strokeColor=self.boxStrokeColor,
+                            strokeWidth=self.boxStrokeWidth,
+                            fillColor=self.boxFillColor)
+                            )
+            g1 = Group()
+            g1.translate(x,self._top-self._eheight)
+            g1.add(self._ddf(self._obj))
+            g.add(g1)
+            return g
