@@ -1,8 +1,8 @@
-#Copyright ReportLab Europe Ltd. 2000-2012
+#Copyright ReportLab Europe Ltd. 2000-2017
 #see license.txt for license details
-#history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/pdfbase/cidfonts.py
+#history https://hg.reportlab.com/hg-public/reportlab/log/tip/src/reportlab/pdfbase/cidfonts.py
 #$Header $
-__version__=''' $Id$ '''
+__version__='3.3.0'
 __doc__="""CID (Asian multi-byte) font support.
 
 This defines classes to represent CID fonts.  They know how to calculate
@@ -449,7 +449,7 @@ def precalculate(cmapdir):
     import os
     files = os.listdir(cmapdir)
     for file in files:
-        if os.path.isfile(cmapdir + os.sep + self.name + '.fastmap'):
+        if os.path.isfile(cmapdir + os.sep + file + '.fastmap'):
             continue
         try:
             enc = CIDEncoding(file)
