@@ -11,6 +11,7 @@ import os, glob, sys, traceback, unittest
 #directory and run this directly
 if __name__=='__main__':
     P=[]
+    os.environ['RL_trustedHosts'] = '*.reportlab.com'
     try:
         from reportlab.lib.testutils import setOutDir
     except ImportError:
@@ -91,9 +92,11 @@ def main(pattern='test_*.py'):
     cleanup(outputfile(''))
     NI = []
     cleanOnly = '--clean' in sys.argv
+    verbosity = [_ for _ in sys.argv if _.startswith('--verbosity=')]
+    verbosity = int(verbosity[-1][12:]) if verbosity else 1
     if not cleanOnly:
         testSuite = makeSuite(folder,nonImportable=NI,pattern=pattern+(not haveSRC and 'c' or ''))
-        result = unittest.TextTestRunner().run(testSuite)
+        result = unittest.TextTestRunner(verbosity=verbosity).run(testSuite)
     else:
         result = None
 
