@@ -5,22 +5,24 @@
 __version__='3.3.0'
 from reportlab.lib.testutils import setOutDir,makeSuiteForClasses, outputfile, printLocation
 setOutDir(__name__)
-import sys, os, time, re
+import sys, os, time, re, random
+from reportlab.rl_config import invariant as rl_invariant
 from operator import truth
 import unittest
-from reportlab.platypus.flowables import Flowable
+from reportlab.platypus.flowables import Flowable, KeepTogether, KeepTogetherSplitAtTop
 from reportlab.lib import colors
-from reportlab.lib.units import cm
+from reportlab.lib.units import cm, mm
 from reportlab.lib.enums import TA_LEFT, TA_RIGHT, TA_CENTER, TA_JUSTIFY
 from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
 from reportlab.platypus.paragraph import Paragraph
 from reportlab.platypus.frames import Frame
 from reportlab.lib.randomtext import randomText, PYTHON
-from reportlab.platypus.doctemplate import PageTemplate, BaseDocTemplate, Indenter, SimpleDocTemplate
+from reportlab.platypus.doctemplate import PageTemplate, BaseDocTemplate, Indenter, SimpleDocTemplate, LayoutError
 from reportlab.platypus.paragraph import *
 from reportlab.pdfgen.canvas import Canvas
-from reportlab.rl_config import invariant, paraFontSizeHeightOffset
+from reportlab.rl_config import paraFontSizeHeightOffset
 from reportlab.pdfbase.pdfmetrics import stringWidth
+from reportlab.lib.pagesizes import A4, portrait
 
 def myMainPageFrame(canvas, doc):
     "The page frame used for all PDF documents."
@@ -60,98 +62,112 @@ subject to the levels of acceptability from fairly high (e.g. (99a)) to
 virtual gibberish (e.g. (98d)).
 '''
 def _test0(self):
-    "This makes one long multi-page paragraph."
+    "This makes one long multi-page paragraph in test_platypus_breaking."
+
+    if rl_invariant: random.seed(1532760416)
+    def RT(k,theme='PYTHON',sentences=1,cache={}):
+        if k not in cache:
+            cache[k] = randomText(theme=theme,sentences=sentences)
+        return cache[k]
 
     # Build story.
-    story = []
-    a = story.append
+    def makeStory():
+        story = []
+        a = story.append
 
-    styleSheet = getSampleStyleSheet()
-    h1 = styleSheet['Heading1']
-    h1.pageBreakBefore = 1
-    h1.keepWithNext = 1
+        styleSheet = getSampleStyleSheet()
+        h1 = styleSheet['Heading1']
+        h1.pageBreakBefore = 1
+        h1.keepWithNext = 1
 
-    h2 = styleSheet['Heading2']
-    h2.frameBreakBefore = 1
-    h2.keepWithNext = 1
+        h2 = styleSheet['Heading2']
+        h2.frameBreakBefore = 1
+        h2.keepWithNext = 1
 
-    h3 = styleSheet['Heading3']
-    h3.backColor = colors.cyan
-    h3.keepWithNext = 1
+        h3 = styleSheet['Heading3']
+        h3.backColor = colors.cyan
+        h3.keepWithNext = 1
 
-    bt = styleSheet['BodyText']
-    btj = ParagraphStyle('bodyText1j',parent=bt,alignment=TA_JUSTIFY)
-    btr = ParagraphStyle('bodyText1r',parent=bt,alignment=TA_RIGHT)
-    btc = ParagraphStyle('bodyText1c',parent=bt,alignment=TA_CENTER)
-    a(Paragraph("""
-        <a name='top'/>Subsequent pages test pageBreakBefore, frameBreakBefore and
-        keepTogether attributes.  Generated at %s.  The number in brackets
-        at the end of each paragraph is its position in the story. (%d)""" % (
-            time.ctime(946684800.0 if invariant else time.time()), len(story)), bt))
+        bt = styleSheet['BodyText']
+        btj = ParagraphStyle('bodyText1j',parent=bt,alignment=TA_JUSTIFY)
+        btr = ParagraphStyle('bodyText1r',parent=bt,alignment=TA_RIGHT)
+        btc = ParagraphStyle('bodyText1c',parent=bt,alignment=TA_CENTER)
+        from reportlab.lib.utils import TimeStamp
+        ts = TimeStamp()
+        a(Paragraph("""
+            <a name='top'/>Subsequent pages test pageBreakBefore, frameBreakBefore and
+            keepTogether attributes.  Generated at %s.  The number in brackets
+            at the end of each paragraph is its position in the story. (%d)""" % (
+                ts.asctime, len(story)), bt))
 
-    for i in range(10):
-        a(Paragraph('Heading 1 always starts a new page (%d)' % len(story), h1))
-        for j in range(3):
-            a(Paragraph('Heading1 paragraphs should always'
-                            'have a page break before.  Heading 2 on the other hand'
-                            'should always have a FRAME break before (%d)' % len(story), bt))
-            a(Paragraph('Heading 2 always starts a new frame (%d)' % len(story), h2))
-            a(Paragraph('Heading1 paragraphs should always'
-                            'have a page break before.  Heading 2 on the other hand'
-                            'should always have a FRAME break before (%d)' % len(story), bt))
+        for i in range(10):
+            a(Paragraph('Heading 1 always starts a new page (%d)' % len(story), h1))
             for j in range(3):
-                a(Paragraph(randomText(theme=PYTHON, sentences=2)+' (%d)' % len(story), bt))
-                a(Paragraph('I should never be at the bottom of a frame (%d)' % len(story), h3))
-                a(Paragraph(randomText(theme=PYTHON, sentences=1)+' (%d)' % len(story), bt))
+                a(Paragraph('Heading1 paragraphs should always'
+                                'have a page break before.  Heading 2 on the other hand'
+                                'should always have a FRAME break before (%d)' % len(story), bt))
+                a(Paragraph('Heading 2 always starts a new frame (%d)' % len(story), h2))
+                a(Paragraph('Heading1 paragraphs should always'
+                                'have a page break before.  Heading 2 on the other hand'
+                                'should always have a FRAME break before (%d)' % len(story), bt))
+                for j in range(3):
+                    a(Paragraph(RT((i,j,0),theme=PYTHON, sentences=2)+' (%d)' % len(story), bt))
+                    a(Paragraph('I should never be at the bottom of a frame (%d)' % len(story), h3))
+                    a(Paragraph(RT((i,j,1),theme=PYTHON, sentences=1)+' (%d)' % len(story), bt))
 
-    for align,bts in [('left',bt),('JUSTIFIED',btj),('RIGHT',btr),('CENTER',btc)]:
-        a(Paragraph('Now we do &lt;br/&gt; tests(align=%s)' % align, h1))
-        a(Paragraph('First off no br tags',h3))
-        a(Paragraph(_text1,bts))
-        a(Paragraph("&lt;br/&gt; after 'the' in line 4",h3))
-        a(Paragraph(_text1.replace('forms of the','forms of the<br/>',1),bts))
-        a(Paragraph("2*&lt;br/&gt; after 'the' in line 4",h3))
-        a(Paragraph(_text1.replace('forms of the','forms of the<br/><br/>',1),bts))
-        a(Paragraph("&lt;br/&gt; after 'I suggested ' in line 5",h3))
-        a(Paragraph(_text1.replace('I suggested ','I suggested<br/>',1),bts))
-        a(Paragraph("2*&lt;br/&gt; after 'I suggested ' in line 5",h3))
-        a(Paragraph(_text1.replace('I suggested ','I suggested<br/><br/>',1),bts))
-        a(Paragraph("&lt;br/&gt; at the end of the paragraph!",h3))
-        a(Paragraph("""text one<br/>text two<br/>""",bts))
-        a(Paragraph("Border with &lt;br/&gt; at the end of the paragraph!",h3))
-        bt1 = ParagraphStyle('bodyText1',bts)
-        bt1.borderWidth = 0.5
-        bt1.borderColor = colors.toColor('red')
-        bt1.backColor = colors.pink
-        bt1.borderRadius = 2
-        bt1.borderPadding = 3
-        a(Paragraph("""text one<br/>text two<br/>""",bt1))
-        a(Paragraph("Border no &lt;br/&gt; at the end of the paragraph!",h3))
-        bt1 = ParagraphStyle('bodyText1',bts)
-        bt1.borderWidth = 0.5
-        bt1.borderColor = colors.toColor('red')
-        bt1.backColor = colors.pink
-        bt1.borderRadius = 2
-        bt1.borderPadding = 3
-        a(Paragraph("""text one<br/>text two""",bt1))
-        a(Paragraph("Different border style!",h3))
-        bt2 = ParagraphStyle('bodyText1',bt1)
-        bt2.borderWidth = 1.5
-        bt2.borderColor = colors.toColor('blue')
-        bt2.backColor = colors.gray
-        bt2.borderRadius = 3
-        bt2.borderPadding = 3
-        a(Paragraph("""text one<br/>text two<br/>""",bt2))
-    for i in 0, 1, 2:
-        P = Paragraph("""This is a paragraph with <font color='blue'><a href='#top'>with an incredibly
+        for align,bts in [('left',bt),('JUSTIFIED',btj),('RIGHT',btr),('CENTER',btc)]:
+            a(Paragraph('Now we do &lt;br/&gt; tests(align=%s)' % align, h1))
+            a(Paragraph('First off no br tags',h3))
+            a(Paragraph(_text1,bts))
+            a(Paragraph("&lt;br/&gt; after 'the' in line 4",h3))
+            a(Paragraph(_text1.replace('forms of the','forms of the<br/>',1),bts))
+            a(Paragraph("2*&lt;br/&gt; after 'the' in line 4",h3))
+            a(Paragraph(_text1.replace('forms of the','forms of the<br/><br/>',1),bts))
+            a(Paragraph("&lt;br/&gt; after 'I suggested ' in line 5",h3))
+            a(Paragraph(_text1.replace('I suggested ','I suggested<br/>',1),bts))
+            a(Paragraph("2*&lt;br/&gt; after 'I suggested ' in line 5",h3))
+            a(Paragraph(_text1.replace('I suggested ','I suggested<br/><br/>',1),bts))
+            a(Paragraph("&lt;br/&gt; at the end of the paragraph!",h3))
+            a(Paragraph("""text one<br/>text two<br/>""",bts))
+            a(Paragraph("Border with &lt;br/&gt; at the end of the paragraph!",h3))
+            bt1 = ParagraphStyle('bodyText1',bts)
+            bt1.borderWidth = 0.5
+            bt1.borderColor = colors.toColor('red')
+            bt1.backColor = colors.pink
+            bt1.borderRadius = 2
+            bt1.borderPadding = 3
+            a(Paragraph("""text one<br/>text two<br/>""",bt1))
+            a(Paragraph("Border no &lt;br/&gt; at the end of the paragraph!",h3))
+            bt1 = ParagraphStyle('bodyText1',bts)
+            bt1.borderWidth = 0.5
+            bt1.borderColor = colors.toColor('red')
+            bt1.backColor = colors.pink
+            bt1.borderRadius = 2
+            bt1.borderPadding = 3
+            a(Paragraph("""text one<br/>text two""",bt1))
+            a(Paragraph("Different border style!",h3))
+            bt2 = ParagraphStyle('bodyText1',bt1)
+            bt2.borderWidth = 1.5
+            bt2.borderColor = colors.toColor('blue')
+            bt2.backColor = colors.gray
+            bt2.borderRadius = 3
+            bt2.borderPadding = 3
+            a(Paragraph("""text one<br/>text two<br/>""",bt2))
+        for i in 0, 1, 2:
+            P = Paragraph("""This is a paragraph with <font color='blue'><a href='#top'>with an incredibly
 long and boring link in side of it that
 contains lots and lots of stupidly boring and worthless information.
 So that we can split the link and see if we get problems like Dinu's.
 I hope we don't, but you never do Know.</a></font>""",bt)
-        a(P)
+            a(P)
+        return story
 
-    doc = MyDocTemplate(outputfile('test_platypus_breaking.pdf'))
-    doc.multiBuild(story)
+    for sfx,klass in (('',KeepTogether),('_ktsat',KeepTogetherSplitAtTop)):
+        doc = MyDocTemplate(outputfile('test_platypus_breaking%s.pdf'%sfx),keepTogetherClass=klass,
+                displayDocTitle=(sfx==''),
+                duplex='Simplex',
+                )
+        doc.multiBuild(makeStory())
 
 class BreakingTestCase(unittest.TestCase):
     "Test multi-page splitting of paragraphs (eyeball-test)."
@@ -383,6 +399,39 @@ class BreakingTestCase(unittest.TestCase):
         canv.save()
         if errors:
             raise ValueError('\n'.join(errors))
+    
+    def test5(self):
+        '''extreme test inspired by Moritz Pfeiffer https://bitbucket.org/moritzpfeiffer/'''
+        with self.assertRaises(LayoutError):
+            text="""
+            Clearly, the natural general principle that will subsume this case is
+            not subject to a parasitic gap construction.  Presumably, most of the
+            methodological work in modern linguistics can be defined in such a way
+            as to impose the system of base rules exclusive of the lexicon.  In the
+            discussion of resumptive pronouns following (81), the fundamental error
+            of regarding functional notions as categorial is to be regarded as a
+            descriptive <span color="red">fact</span>.<br/>So far, the earlier discussion of deviance is not
+            quite equivalent to a parasitic gap construction.  To characterize a
+            linguistic level L, a case of semigrammaticalness of a different sort
+            may remedy and, at the same time, eliminate irrelevant intervening
+            contexts in selectional <span color="red">rules</span>.<br/>
+            Summarizing, then, we assume that the descriptive power of the base
+            component can be defined in such a way as to impose nondistinctness in
+            the sense of distinctive feature theory.
+            """
+            styleSheet = getSampleStyleSheet()
+            story = []
+            story.append(Paragraph(text, styleSheet['Normal']))
+            doc = BaseDocTemplate(
+                outputfile('test_platypus_much_too_large.pdf'),
+                pagesize=portrait(A4),
+                pageTemplates=[PageTemplate(
+                    'page_template',
+                    [Frame(0, 0, 0, 0, leftPadding=0, rightPadding=0, topPadding=0, bottomPadding=0, id='DUMMY_FRAME')],
+                    )],
+                )
+            doc.build(story)
+
 
 def makeSuite():
     return makeSuiteForClasses(BreakingTestCase)
